clear all
% =====================================================================
%   < System Wide Runs >
%       : in preparation for submission to JMCB
%    
%   - by DongIk Kang
%   - last update : 2022/03/21
% =====================================================================

% =====================================================================
% (0) Parameter Values
% =====================================================================
delta = 0.1;          
aH = 1.2;              
phi = 0.8;             
aL = phi*aH;         
dlimit = 1.05;        

% distribution of returns (lognormal distribution)
mu = -0.08;                                       
sigma = 0.4;                                     
mean = exp(mu + sigma^2/2);                      
variance = (exp(sigma^2)-1)*exp(2*mu+sigma^2);   
F = @(x) logncdf(x,mu,sigma);
f = @(x) lognpdf(x,mu,sigma);

% Marginal return for investors
I = @(x) 20*exp(-4*x)+0.01;



%% 1. Case with credit contraction
B = 3.2;               % bank borrowing capacity (contraction)
% =====================================================================
% (1) No run
% =====================================================================
% 1. Fire sales prices 
rhomfind = @(x) delta*(1/x - 1) - I(B-x*phi*aH);
rho_m = fsolve(rhomfind ,1);
    
% 2. Liquidation cutoffs (condition 1)
d0_m = 1;
liq_cutm = d0_m/aH;
    
% 3. Debt floor (condition 2)
Rf = @(x) x.*f(x);
Rfind = @(x) rho_m - integral(Rf,0,x);
Rstar_m = fsolve(Rfind ,1);
dfloorH_m = (d0_m-rho_m*aH)/(1-F(Rstar_m));
dfloorL_m = (d0_m-rho_m*aL)/(1-F(Rstar_m));


% =====================================================================
% (1) Run
% =====================================================================
% 1. Fire sales prices 
rhosfind = @(x) delta*(1/x - 1) - I(B-x*aH);
rho_s = fsolve(rhosfind ,1);
   
% 2. Liquidation cutoffs (condition 1)
d0_s = (1-delta*rho_s*aH)/(1-delta);      % period 0 debt if fire sale is severe
liq_cuts = d0_s/aH;
    
% 3. Debt floor (condition 2)
Rf = @(x) x.*f(x);
Rfind = @(x) rho_s - integral(Rf,0,x);
Rstar_s = fsolve(Rfind ,1);
dfloorH_s = (d0_s-rho_s*aH)/(1-F(Rstar_s));
dfloorL_s = (d0_s-rho_s*aL)/(1-F(Rstar_s));


disp('=========================================================================')
fprintf('aH = %.4f, delta = %.4f, phi = %.4f, aL = %.4f\n', aH, delta, phi, aL); 
fprintf('dlimit = %.4f, B = %.4f\n', dlimit, B); 
fprintf('I(beta) = %.4fexp(%.4f beta)+ %.4f\n', 20, -4, 0.01); 
disp('=========================================================================')

disp('=========================================================================')
fprintf('rho_s = %.4f, rho_m = %.4f\n', rho_s, rho_m ); 
fprintf('Rstar_s = %.4f,   Rstar_m =  %.4f\n', Rstar_s, Rstar_m); 
fprintf('dfloorH_s = %.4f,  dfloorL_s = %.4f,  dfloorH_m = %.4f,  dfloorL_m = %.4f\n', dfloorH_s, dfloorL_s, dfloorH_m, dfloorL_m); 
fprintf('d0_m = %.4f,  q0_m = %.4f,  beta_s = %.4f\n', 1, 1, B-rho_s*aH); 
disp('=========================================================================')

disp('=========================================================================')
fprintf('Liquidation value (mild): rho aH - d0 = %.4f\n', rho_m*aH-d0_m); 
fprintf('Condition 1: dfloorH < (1-phi)aHRstar <=> %.4f < %.4f\n', dfloorH_m, (1-phi)*aH*Rstar_m); 
fprintf('Liquidation value (severe): rho aH - d0 = %.4f\n', rho_s*aH-d0_s); 
fprintf('Condition 2: dfloorH > (1-phi)aHRstar <=> %.4f > %.4f\n', dfloorH_s, (1-phi)*aH*Rstar_s); 
disp('=========================================================================')



%% 2. Case with system wide runs
B = 2.4;               % bank borrowing capacity (system wide run)
% =====================================================================
% (1) Run
% =====================================================================
% 1. Fire sales prices 
rhosfind = @(x) delta*(1/x - 1) - I(B-x*aH);
rho_s = fsolve(rhosfind ,1);
   
% 2. Liquidation cutoffs (condition 1)
d0_s = (1-delta*rho_s*aH)/(1-delta);      % period 0 debt if fire sale is severe
liq_cuts = d0_s/aH;
    
% 3. Debt floor (condition 2)
Rf = @(x) x.*f(x);
Rfind = @(x) rho_s - integral(Rf,0,x);
Rstar_s = fsolve(Rfind ,1);
dfloorH_s = (d0_s-rho_s*aH)/(1-F(Rstar_s));
dfloorL_s = (d0_s-rho_s*aL)/(1-F(Rstar_s));

% =====================================================================
% (2) No run
% =====================================================================
% 1. Fire sales prices 
rhomfind = @(x) delta*(1/x - 1) - I(B-x*phi*aH);
rho_m = fsolve(rhomfind ,1);
    
% 2. Liquidation cutoffs (condition 1)
d0_m = 1;
liq_cutm = d0_m/aH;
    
% 3. Debt floor (condition 2)
Rf = @(x) x.*f(x);
Rfind = @(x) rho_m - integral(Rf,0,x);
Rstar_m = fsolve(Rfind ,1);
dfloorH_m = (d0_m-rho_m*aH)/(1-F(Rstar_m));
dfloorL_m = (d0_m-rho_m*aL)/(1-F(Rstar_m));

disp('=========================================================================')
fprintf('aH = %.4f, delta = %.4f, phi = %.4f, aL = %.4f\n', aH, delta, phi, aL); 
fprintf('dlimit = %.4f, B = %.4f\n', dlimit, B); 
fprintf('I(beta) = %.4fexp(%.4f beta)+ %.4f\n', 20, -4, 0.01); 
disp('=========================================================================')

disp('=========================================================================')
fprintf('rho_s = %.4f, rho_m = %.4f\n', rho_s, rho_m ); 
fprintf('Rstar_s = %.4f,   Rstar_m =  %.4f\n', Rstar_s, Rstar_m); 
fprintf('dfloorH_s = %.4f,  dfloorL_s = %.4f,  dfloorH_m = %.4f,  dfloorL_m = %.4f\n', dfloorH_s, dfloorL_s, dfloorH_m, dfloorL_m); 
fprintf('d0_s = %.4f,  q0_s = %.4f,  beta_s = %.4f\n', d0_s, 1/d0_s, B-rho_s*aH); 
disp('=========================================================================')

disp('=========================================================================')
fprintf('Condition 1: dfloorH < (1-phi)aHRstar <=> %.4f < %.4f\n', dfloorH_m, (1-phi)*aH*Rstar_m); 
fprintf('Condition 2: dfloorH > (1-phi)aHRstar <=> %.4f > %.4f\n', dfloorH_s, (1-phi)*aH*Rstar_s); 
disp('=========================================================================')















